----------------------------------------------------------------
--	Developments
----------------------------------------------------------------

----------------------------------------------------------------
--	Library
----------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("GridStatusRaidDebuff")
local BWL = AceLibrary("AceLocale-2.2"):new("BigWigs")
local Aura = AceLibrary("SpecialEvents-Aura-2.0")
local LBZ = LibStub("LibBabble-Zone-3.0"):GetLookupTable()
local BB = LibStub("LibBabble-Boss-3.0"):GetLookupTable()
local RL = AceLibrary("Roster-2.1")
local Version = "2.04d"
----------------------------------------------------------------
--	Variables
----------------------------------------------------------------
local zone
local bossmod_activated = {}
local player_class
local bit_band = bit.band
local select = select

-- 8-Curse / 4-Disease / 2-Poison / 1-Magic
local DispelFlgTable = {
	["SHAMAN"] = 0x7,
	["DRUID"] = 0xA,
	["MAGE"] = 0x8,
	["PRIEST"] = 0x5,
	["PALADIN"] = 0x7,
	["WARLOCK"] = 0x0,
	["WARRIOR"] = 0x0,
	["ROGUE"] = 0x0,
	["HUNTER"] = 0x0,
}
--Flag :  [0][0/1-PRIEST][ 8-ROGUE/4-WARRIOR/2-SHAMAN/1-PALADIN ][8-MAGE/4-WARLOCK/2-HUNTER/1-DRUID]
local CFFlgTable = {
	["PRIEST"] = 0x0100,
	["ROGUE"] = 0x0080,
	["WARRIOR"] = 0x0040,
	["SHAMAN"] = 0x0020,
	["PALADIN"] = 0x0010,
	["MAGE"] = 0x0008,
	["WARLOCK"] = 0x0004,
	["HUNTER"] = 0x0002,
	["DRUID"] = 0x0001,
}


----------------------------------------------------------------
--	Module
----------------------------------------------------------------
GridStatusRaidDebuff = GridStatus:NewModule("GridStatusRaidDebuff")

local GridStatusRaidDebuff = GridStatusRaidDebuff
GridStatusRaidDebuff.menuName = L["Raid Debuff"]

GridStatusRaidDebuff.options = false

GridStatusRaidDebuff.defaultDB = {
	debug = false,

	pos_dispel_toggle = true,
	imp_dispel_toggle = true,
	dose_toggle_rd = true,
	dose_toggle_ct = true,

	version = "2.0",

	alert_RaidDebuff_24 = {
		text = L["Raid Debuff"],
		desc = L["Raid Debuff"],
		enable = true,
		color = { r = 1.0, g = 0.0, b = 0.0, a = 1 },
		priority = 99,
		range = false,

		order_list = {
			[LBZ["Karazhan"]] = 11,
			[LBZ["Zul'Aman"]] = 12,
			[LBZ["Serpentshrine Cavern"]] = 13,
			[LBZ["Tempest Keep"]] = 14,
			[LBZ["Hyjal Summit"]] = 15,
			[LBZ["Black Temple"]] = 16,
			[LBZ["Sunwell Plateau"]] = 17,
		},
		debuff_list = {
			[LBZ["Karazhan"]] = {
				["Garrote"] = {
					enable = true,
					owner = BB["Moroes"],
					debuffIndex = 37066,
					cf = 0xffff,
					order = 1,
					dose = false,
					debuffType = 0x0,
				},
				--[[ Need Infomation ***
				["Blind"] = {
					enable = true,
					owner = BB["Moroes"],
					debuffIndex = 6788,
					cf = 0xffff,
					order = 2,
					dose = false,
					debuffType = 0x2,
				},
				["Gouge"]= {
					enable = true,
					owner = BB["Moroes"],
					debuffIndex = 6788,
					cf = 0xffff,
					order = 3,
					dose = false,
					debuffType = 0x0,
				},
				]]
				["Holy Fire"] = {
					enable = true,
					owner = BB["Maiden of Virtue"],
					debuffIndex = 29522,
					cf = 0xffff,
					order = 4,
					dose = false,
					debuffType = 0x1,
				},
				["Repentance"] = {
					enable = true,
					owner = BB["Maiden of Virtue"],
					debuffIndex = 29511,
					cf = 0xffff,
					order = 5,
					dose = false,
					debuffType = 0x0,
				},
				["Red Riding Hood"] = {
					enable = true,
					owner = BB["The Big Bad Wolf"],
					debuffIndex = 30753,
					cf = 0xffff,
					order = 6,
					dose = false,
					debuffType = 0x0,
				},
				["Sacrifice"] = {
					enable = true,
					owner = BB["Terestian Illhoof"],
					debuffIndex = 30115,
					cf = 0xffff,
					order = 7,
					dose = false,
					debuffType = 0x0,

				},
				["Enfeeble"]= {
					enable = true,
					owner = BB["Prince Malchezaar"],
					debuffIndex = 30843, --check***
					cf = 0xffff,
					order = 8,
					dose = false,
					debuffType = 0x0,
				},
			},
			[LBZ["Zul'Aman"]] = {
				["Mangle"] = {
					enable = true,
					owner = BB["Nalorakk"],
					debuffIndex = 44955,
					cf = 0xffff,
					order = 1,
					dose = false,
					debuffType = 0x0,
				},
				["Electrical Storm"] = {
					enable = true,
					owner = BB["Akil'zon"],
					debuffIndex = 43657,
					order = 2,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Static Distruption"] = {
					enable = true,
					owner = BB["Akil'zon"],
					debuffIndex = 43622,
					order = 3,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Flame Buffet"] = {
					enable = true,
					owner = BB["Jan'alai"],
					debuffIndex = 43299, --check***
					order = 4,
					cf = 0xffff,
					dose = true,
					color = { r = 1.0, g = 0.0, b = 0.0, a = 1 },
					maxCount = 20,
					debuffType = 0x1,
				},
				["Flame Shock"] = {
					enable = true,
					owner = BB["Halazzi"],
					debuffIndex = 43303,
					order = 5,
					cf = 0xffff,
					dose = false,
					debuffType = 0x1,
				},
				["Cold Stare"] = {
					enable = true,
					owner = BB["Hex Lord Malacrass"],
					debuffIndex = 43613,
					order = 6,
					cf = 0xffff,
					dose = false,
					debuffType = 0x8,
				},
				["Siphon Soul"] = {
					enable = true,
					owner = BB["Hex Lord Malacrass"],
					debuffIndex = 43501,
					order = 7,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Grievous Throw"] = {
					enable = true,
					owner = BB["Zul'jin"],
					debuffIndex = 43093,
					order = 8,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Creeping Paralysis"] = {
					enable = true,
					owner = BB["Zul'jin"],
					debuffIndex = 43095,
					order = 9,
					cf = 0xffff,
					dose = false,
					debuffType = 0x1,
				},
				["Claw Rage"] = {
					enable = true,
					owner = BB["Zul'jin"],
					debuffIndex = 43150,
					order = 10,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
			},
			[LBZ["Serpentshrine Cavern"]] = {
				["Rampant Infection"] = {
					enable = true,
					owner = L["Trash Mob"],
					debuffIndex = 39042,
					order = 1,
					cf = 0xffff,
					dose = false,
					debuffType = 0x4,
				},
				["Serpentshrine Parasite"] = {
					enable = true,
					owner = L["Trash Mob"],
					debuffIndex = 39044, --or 39053 ***
					order = 2,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Water Tomb"] = {
					enable = true,
					owner = BB["Hydross the Unstable"],
					debuffIndex = 38235,
					order = 3,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Vile Sludge"] = {
					enable = true,
					owner = BB["Hydross the Unstable"],
					debuffIndex = 38246,
					order = 4,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Watery Grave"] = {
					enable = true,
					owner = BB["Morogrim Tidewalker"],
					debuffIndex = 38049,
					order = 5,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Insidious Whisper"] = {
					enable = true,
					owner = BB["Leotheras the Blind"],
					debuffIndex = 37676,
					order = 6,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Whirlwind"] = {
					enable = true,
					owner = BB["Leotheras the Blind"],
					debuffIndex = 37640,
					order = 7,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Consuming Madness"] = {
					enable = true,
					owner = BB["Leotheras the Blind"],
					debuffIndex = 37749,
					order = 8,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Static Charge"] = {
					enable = true,
					owner = BB["Lady Vashj"],
					debuffIndex = 38280,
					order = 9,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
			},
			--The Eye
			[LBZ["Tempest Keep"]] = {
				["Saw Blade"] = {
					enable = true,
					owner = L["Trash Mob"],
					debuffIndex = 35318, --check***
					order = 1,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Fragmentation Bomb"] = {
					enable = true,
					owner = L["Trash Mob"],
					debuffIndex = 37120,
					order = 2,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Shell Shock"] = {
					enable = true,
					owner = L["Trash Mob"],
					debuffIndex = 37118,
					order = 3,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Wrath of the Astromancer"] = {
					enable = true,
					owner = BB["High Astromancer Solarian"],
					debuffIndex = 42783,
					order = 4,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Remote Toy"] = {
					enable = true,
					owner = BB["Kael'thas Sunstrider"],
					debuffIndex = 37027,
					order = 5,
					cf = 0x0051,
					dose = false,
					debuffType = 0x0,
				},
				["Mind Control"] = {
					enable = true,
					owner = BB["Kael'thas Sunstrider"],
					debuffIndex = 36798,
					order = 6,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
			},
			[LBZ["Hyjal Summit"]] = {
				["Icebolt"] = {
					enable = true,
					owner = BB["Rage Winterchill"],
					debuffIndex = 31249,
					order = 1,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Carrion Swarm"] = {
					enable = true,
					owner = BB["Anetheron"],
					debuffIndex = 31306,
					order = 2,
					cf = 0x0131,
					dose = false,
					debuffType = 0x0,
				},
				["Sleep"] = {
					enable = true,
					owner = BB["Anetheron"],
					debuffIndex = 31298, --check ***
					order = 3,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Doom"] = {
					enable = true,
					owner = BB["Azgalor"],
					debuffIndex = 31347,
					order = 4,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Howl of Azgalor"] = {
					enable = true,
					owner = BB["Azgalor"],
					debuffIndex = 31344,
					order = 5,
					cf = 0x0131,
					dose = false,
					debuffType = 0x0,
				},
				["Unquenchable Flames"] = {
					enable = true,
					owner = BB["Azgalor"],
					debuffIndex = 31341,
					order = 7,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Doomfire"] = {
					enable = true,
					owner = BB["Archimonde"],
					debuffIndex = 31944, --check ***
					order = 8,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Grip of the Legion"] = {
					enable = true,
					owner = BB["Archimonde"],
					debuffIndex = 31972,
					order = 9,
					cf = 0xffff,
					dose = false,
					debuffType = 0x8,
				},
			},
			[LBZ["Black Temple"]] = {
				["Impaling Spine"] = {
					enable = true,
					owner = BB["High Warlord Naj'entus"],
					debuffIndex = 39837,
					order = 1,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Debilitating Strike"] = {
					enable = true,
					owner = BB["Shade of Akama"],
					debuffIndex = 41178,--41179?
					order = 2,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Debilitating Poison"] = {
					enable = true,
					owner = BB["Shade of Akama"],
					debuffIndex = 41978,
					order = 3,
					cf = 0xffff,
					dose = false,
					debuffType = 0x2,
				},
				["Incinerate"] = {
					enable = true,
					owner = BB["Teron Gorefiend"],
					debuffIndex = 40239, --check***
					order = 4,
					cf = 0xffff,
					dose = false,
					debuffType = 0x1,
				},
				["Shadow of Death"] = {
					enable = true,
					owner = BB["Teron Gorefiend"],
					debuffIndex = 40251,
					order = 5,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Crushing Shadows"] = {
					enable = true,
					owner = BB["Teron Gorefiend"],
					debuffIndex = 40243,
					order = 6,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Fel Rage"] = {
					enable = true,
					owner = BB["Gurtogg Bloodboil"],
					debuffIndex = 40604,
					order = 7,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Acidic Wound"] = {
					enable = true,
					owner = BB["Gurtogg Bloodboil"],
					debuffIndex = 40481,
					order = 8,
					cf = 0xffff,
					dose = true,
					color = { r = 0.0, g = 1.0, b = 0.0, a = 1 },
					maxCount = 15,
					debuffType = 0x0,
				},
				["Fel-Acid Breath"] = {
					enable = true,
					owner = BB["Gurtogg Bloodboil"],
					debuffIndex = 40508, --40595 ?
					order = 9,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Bloodboil"] = {
					enable = true,
					owner = BB["Gurtogg Bloodboil"],
					debuffIndex = 42005,
					order = 10,
					cf = 0xffff,
					dose = true,
					color = { r = 1.0, g = 0.0, b = 0.0, a = 1 },
					maxCount = 3,
					debuffType = 0x0,
				},
				["Soul Drain"] = {
					enable = true,
					owner = BB["Reliquary of Souls"],
					debuffIndex = 41303,
					order = 11,
					cf = 0xffff,
					dose = false,
					debuffType = 0x1,
				},
				["Deaden"] = {
					enable = true,
					owner = BB["Reliquary of Souls"],
					debuffIndex = 41410,
					order = 12,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},

				["Vile Beam"] = {
					enable = true,
					owner = BB["Mother Shahraz"],
					debuffIndex = 40860,
					order = 13,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Fatal Attraction"] = {
					enable = true,
					owner = BB["Mother Shahraz"],
					debuffIndex = 41001,
					order = 14,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Deadly Poison"] = {
					enable = true,
					owner = BB["The Illidari Council"],
					debuffIndex = 41485,
					order = 15,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Divine Wrath"] = {
					enable = true,
					owner = BB["The Illidari Council"],
					debuffIndex = 41472,
					order = 16,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Parasitic Shadowfiend"] = {
					enable = true,
					owner = BB["Illidan Stormrage"],
					debuffIndex = 41914,
					order = 17,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Dark Barrage"] = {
					enable = true,
					owner = BB["Illidan Stormrage"],
					debuffIndex = 40585,
					order = 18,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Shear"] = {
					enable = true,
					owner = BB["Illidan Stormrage"],
					debuffIndex = 41032,
					order = 19,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Agonizing Flames"] = {
					enable = true,
					owner = BB["Illidan Stormrage"],
					debuffIndex = 40932,
					order = 20,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
			},
			[LBZ["Sunwell Plateau"]] = {
				["Fear"] = {
					enable = true,
					owner = L["Trash Mob"],
					debuffIndex = 46561,
					order = 1,
					cf = 0xffff,
					dose = false,
					debuffType = 0x1,
				},
				["Mind Flay"] = {
					enable = true,
					owner = L["Trash Mob"],
					debuffIndex = 46562,
					order = 2,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Burn Mana"] = {
					enable = true,
					owner = L["Trash Mob"],
					debuffIndex = 46266,
					order = 3,
					cf = 0xffff,
					dose = false,
					debuffType = 0x1,
				},
				["Slaying Shot"] = {
					enable = true,
					owner = L["Trash Mob"],
					debuffIndex = 46557,
					order = 4,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Shadow Word: Pain"] = {
					enable = true,
					owner = L["Trash Mob"],
					debuffIndex = 46560,
					order = 5,
					cf = 0xffff,
					dose = false,
					debuffType = 0x1,
				},
				["Curse of Boundless Agony"] = {
					enable = true,
					owner = BB["Kalecgos"],
					debuffIndex = 45032,
					order = 10,
					cf = 0xffff,
					dose = false,
					debuffType = 0x8,
				},
				["Spectral Realm"] = {
					enable = true,
					owner = BB["Kalecgos"],
					debuffIndex = 46021,
					order = 11,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Arcane Buffet"] = {
					enable = true,
					owner = BB["Kalecgos"],
					debuffIndex = 45018,
					order = 12,
					cf = 0xffff,
					dose = true,
					color = { r = 1.0, g = 0.0, b = 1.0, a = 1 },
					maxCount = 10,
					debuffType = 0x0,
				},
				["Burn"] = {
					enable = true,
					owner = BB["Brutallus"],
					debuffIndex = 46394,
					order = 20,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Meteor Slash"] = {
					enable = true,
					owner = BB["Brutallus"],
					debuffIndex = 45150,
					order = 21,
					cf = 0xffff,
					dose = true,
					color = { r = 1.0, g = 0.0, b = 0.0, a = 1 },
					maxCount = 10,
					debuffType = 0x0,
				},
				["Gas Nova"] = {
					enable = true,
					owner = BB["Felmyst"],
					debuffIndex = 45855,
					order = 30,
					cf = 0xffff,
					dose = false,
					debuffType = 0x1,
				},
				["Encapsulate"] = {
					enable = true,
					owner = BB["Felmyst"],
					debuffIndex = 45662,
					order = 31,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Fog of Corruption"] = {
					enable = true,
					owner = BB["Felmyst"],
					debuffIndex = 45717,
					order = 32,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Dark Strike"] = {
					enable = true,
					owner = BB["The Eredar Twins"],
					debuffIndex = 45256,
					order = 40,
					cf = 0xffff,
					dose = true,
					maxCount = 4,
					debuffType = 0x0,
					color = { r = 1.0, g = 0.0, b = 1.0, a = 1 },
				},
				["Confounding Blow"] = {
					enable = true,
					owner = BB["The Eredar Twins"],
					debuffIndex = 45256,
					order = 41,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Conflagration"] = {
					enable = true,
					owner = BB["The Eredar Twins"],
					debuffIndex = 45333,
					order = 42,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Flame Sear"] = { --Restored
					enable = true,
					owner = BB["The Eredar Twins"],
					debuffIndex = 46771,
					order = 43,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Shadowfury"] = {
					enable = true,
					owner = BB["The Eredar Twins"],
					debuffIndex = 45270,
					order = 44,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},
				["Dark Touched"] = {
					enable = true,
					owner = BB["The Eredar Twins"],
					debuffIndex = 45347,
					order = 45,
					cf = 0xffff,
					dose = true,
					maxCount = 20,
					debuffType = 0x0,
					color = { r = 0.2, g = 0.2, b = 0.2, a = 1 },
				},
				["Fire Touched"] = {
					enable = true,
					owner = BB["The Eredar Twins"],
					debuffIndex = 45348,
					order = 46,
					cf = 0xffff,
					dose = true,
					maxCount = 20,
					debuffType = 0x0,
					color = { r = 1.0, g = 0.0, b = 0.0, a = 1 },
				},
				["Darkness"] = {
					enable = true,
					owner = BB["M'uru"],
					debuffIndex = 45996,
					order = 50,
					cf = 0xffff,
					dose = false,
					debuffType = 0x0,
				},

			},
		}
	}
}

function GridStatusRaidDebuff:OnInitialize()
	self:PreLoader()
	self.super.OnInitialize(self)
	self:RegisterStatuses()
end

--for get name and icon(Destination is when user played first time... he can see the debuff tooltip.. this is some problem)
function GridStatusRaidDebuff:PreLoader()
	player_class = select(2,UnitClass("player"))
	for _, cont in pairs(GridStatusRaidDebuff.defaultDB["alert_RaidDebuff_24"]["debuff_list"]) do
		for _, dcont in pairs(cont) do
			local link = GetSpellLink(dcont.debuffIndex)
			if link then
				local link_name,_,link_icon = GetSpellInfo(dcont.debuffIndex)

				dcont.name = link_name
				dcont.link = link
				dcont.icon = dcont.icon or link_icon
			end
		end
	end
end

function GridStatusRaidDebuff:OnEnable()
	if self.db then
		if self.db.profile.version ~= Version then
			self.db.profile.alert_RaidDebuff_24.debuff_list = self.defaultDB.alert_RaidDebuff_24.debuff_list
			self.db.profile.version = Version
		end
	end

	self.debugging = self.db.profile.debug

	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneCheck")
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "UpdateAllUnits")
	self:RegisterEvent("PLAYER_REGEN_DISABLED", "UpdateAllUnits")
	self:RegisterEvent("PLAYER_ALIVE", "UpdateAllUnits")
	self:RegisterEvent("PLAYER_DEAD", "UpdateAllUnits")
	self:RegisterEvent("PLAYER_UNGHOST", "UpdateAllUnits")
	self:RegisterEvent("Grid_UnitJoined")
	self:RegisterEvent("Ace2_AddonEnabled")
	self:RegisterEvent("Ace2_AddonDisabled")

	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "EventHandler")
	self:RegisterEvent("SpecialEvents_UnitDebuffLost", "UpdateUnit")
end

function GridStatusRaidDebuff:ZoneCheck()
	zone = GetRealZoneText()
	if zone:len() > 1 then
		self:UpdateAllUnits()
	end
end

function GridStatusRaidDebuff:Ace2_AddonEnabled(module)
	bossname = module and module.IsBossModule and module:IsBossModule() and module.name
	if bossname then
		bossmod_activated[bossname] = true
	end
end

function GridStatusRaidDebuff:Ace2_AddonDisabled(module)
	bossname = module and module.IsBossModule and module:IsBossModule() and module.name
	if bossname then
		bossmod_activated[bossname] = nil
	end
end

function GridStatusRaidDebuff:PLAYER_ENTERING_WORLD()
	if not zone then
		self:ZoneCheck()
	end
	self:UpdateAllUnits()

end

function GridStatusRaidDebuff:Grid_UnitJoined(name, unitid)
	self:Debug("Joined",name)
	self:UpdateUnit(unitid)
end

function GridStatusRaidDebuff:RegisterStatuses()
	self:RegisterStatus("alert_RaidDebuff_24",L["Raid Debuff"],GridStatusRaidDebuff:CreateDebuffZoneList(),true)
end

function GridStatusRaidDebuff:UnregisterStatuses()
	self:UnregisterStatus("alert_RaidDebuff_24",L["Raid Debuff"])
end

function GridStatusRaidDebuff:CreateDebuffZoneList()
	local debuff_list = self.db.profile["alert_RaidDebuff_24"].debuff_list
	local debuff_menu = {}

	debuff_menu["Zone Header"] = {
		type = "header",
		name = L["Zone Name"],
		order = 1
	}
	debuff_menu["Space1"] = {
		type = "header",
		order = 20
	}
	debuff_menu["possible dispel clear"] = {
		type = "toggle",
		name = L["Show debuff that dispel possible"],
		desc = L["Enable/Disable debuff enabled option that dispel possible(Except |cffff0000\'Can Stack\'|r)"],
		order = 21,
		get = function() return self.db.profile.pos_dispel_toggle end,
		set = function(v)
			self.db.profile.pos_dispel_toggle = v
			for zid, zcont in pairs(debuff_list) do
				for did, dcont in pairs(zcont) do
					if bit_band(dcont.debuffType, DispelFlgTable[player_class]) > 0 and not dcont.dose then
						dcont.enable = v
					end
				end
			end
		end,
	}
	debuff_menu["impossible dispel clear"] = {
		type = "toggle",
		name = L["Show debuff that dispel impossible"],
		desc = L["Enable/Disable debuff enabled option that dispel impossible(Except |cffff0000\'Can Stack\'|r)"],
		order = 22,
		get = function() return self.db.profile.imp_dispel_toggle end,
		set = function(v)
			self.db.profile.imp_dispel_toggle = v
			for zid, zcont in pairs(debuff_list) do
				for did, dcont in pairs(zcont) do
					if bit_band(dcont.debuffType, bit.bnot(DispelFlgTable[player_class])) > 0 and not dcont.dose then
						dcont.enable = v
					end
				end
			end
		end,
	}
	debuff_menu["DoseOption"] = {
		type = "group",
		name = L["Stack Debuff Option"],
		desc = L["Options for Stack Debuff"],
		order= 23,
		args={
			["Show on Raid Debuff"] = {
				type = "toggle",
				name = L["Raid debuff Status"],
				desc = L["Show on raid debuff status"],
				order = 1,
				get = function() return self.db.profile.dose_toggle_rd end,
				set = function(v)
					self.db.profile.dose_toggle_rd = v
				end,
			},
			["Show on Counter"] = {
				type = "toggle",
				name = L["Debuff Counter Indicator"],
				desc = L["Show on debuff counter indicator"],
				order = 2,
				get = function() return self.db.profile.dose_toggle_ct end,
				set = function(v)
					self.db.profile.dose_toggle_ct = v
				end,
			},
		}
	}
	debuff_menu["Space2"] = {
		type = "header",
		order = 30
	}
	debuff_menu["Desc Header"] = {
		type = "toggle",
		name = L["About..."], --need localization
		desc = L["About Desc"],
		order = 99,
		set = function () end,
		get = function () return false end,
	}

	for debuffZone, debuffZoneInfo in pairs(debuff_list) do
		local list = {}

		for debuffName, debuffInfo in pairs(debuffZoneInfo) do
			list[debuffName] = GridStatusRaidDebuff:CreateDebuffList(debuffInfo, debuffZone, debuffName)
		end

		debuff_menu[debuffZone] = {
			type = "group",
			name = debuffZone,
			desc = L["Raid Debuff"].." : "..debuffZone,
			order = self.db.profile["alert_RaidDebuff_24"].order_list[debuffZone],
			args = list
		}
	end

	return debuff_menu
end

function GridStatusRaidDebuff:CreateDebuffList(debuffInfo, debuffZone, debuffName)

	local name_predix = "|r"
	local check_box = tex_no
	local descText = L["Type : "]..L["Normal"]

	if debuffInfo.enable then
		check_box = tex_ok
	end
	
	if bit_band(debuffInfo.debuffType, 0x1) > 0 then --magic
		name_predix = "|cff3f8fff"
		descText = L["Type : "]..L["Magic"]
	elseif bit_band(debuffInfo.debuffType, 0x2) > 0 then --poison
		name_predix = "|cff008f00"
		descText = L["Type : "]..L["Poison"]
	elseif bit_band(debuffInfo.debuffType, 0x4) > 0 then --disease
		name_predix = "|cff8f5f00"
		descText = L["Type : "]..L["Disease"]
	elseif bit_band(debuffInfo.debuffType, 0x8) > 0 then -- curse
		name_predix = "|cff8f00ff"
		descText = L["Type : "]..L["Curse"]
	end

	local name_append = ""

	if debuffInfo.dose then
		name_append = " |cffff0000(+)"
		descText = descText.."\n|cffff0000"..L["(Can Stack)"]
	end

	local nameText = "|T"..debuffInfo.icon..":0|t "..name_predix..debuffInfo.owner.." : "..debuffInfo.name..name_append

	return {
		type = "toggle",
		name = nameText,
		desc = "\n|r"..descText.."\n\n|cffffffff"..L["DescShift"].."\n"..L["DescCtrl"],
		order = debuffInfo.order,
		get = function() return debuffInfo.enable end,
		set = function(v)
			if IsShiftKeyDown() then
				SetItemRef(debuffInfo.link,debuffInfo.link)
			elseif IsControlKeyDown() then
				GridStatus.options.args.alert_RaidDebuff_24.args[debuffZone].args[debuffName].desc = self:GetLinkTooltip(debuffInfo.link).."\n\n|r"..descText.."\n\n|cffffffff"..L["DescShift"]
			else
				debuffInfo.enable = v
			end
		end,
	}
end

function GridStatusRaidDebuff:Reset()
	self.super.Reset(self)
	self:UnregisterStatuses()
	self:RegisterStatuses()
end

function GridStatusRaidDebuff:UpdateAllUnits()
	self:Debug("updating all units")
	for u in RL:IterateRoster() do
		self:UpdateUnit(u.unitid)
	end
end

local function is_active(owner)
	return owner == L["Trash Mob"] or bossmod_activated[owner]
end

function GridStatusRaidDebuff:StatusSender(unitName, debuff, doseDebuff, doseCount, priority, color)
	self:Debug("StatusSender",unitName)

	if debuff == nil and doseDebuff == nil then
		self.core:SendStatusLost(unitName, "alert_RaidDebuff_24")
		self.core:SendStatusLost(unitName, "alert_RaidDebuff_Dose")
		return
	end

	if doseDebuff ~= nil then
		if debuff ~= nil then
			self.core:SendStatusGained(unitName, "alert_RaidDebuff_24", priority, (debuff.range and 40), color, debuff.name, count, nil, debuff.icon)
			if self.db.profile.dose_toggle_ct then
				self.core:SendStatusGained(unitName, "alert_RaidDebuff_Dose", priority, (doseDebuff.range and 40), doseDebuff.color, doseDebuff.name, doseCount, doseDebuff.maxCount, doseDebuff.icon)
			end
		else
			if self.db.profile.dose_toggle_rd then
				self.core:SendStatusGained(unitName, "alert_RaidDebuff_24", priority, (doseDebuff.range and 40), color, doseDebuff.name, 1, nil, doseDebuff.icon)
			end
			if self.db.profile.dose_toggle_ct then
				self.core:SendStatusLost(unitName, "alert_RaidDebuff_24")
				self.core:SendStatusGained(unitName, "alert_RaidDebuff_Dose", priority, (doseDebuff.range and 40), doseDebuff.color, doseDebuff.name, doseCount, doseDebuff.maxCount, doseDebuff.icon)
			end
		end
	else
		self.core:SendStatusGained(unitName, "alert_RaidDebuff_24", priority, (debuff.range and 40), color, debuff.name, count, nil, debuff.icon)
		self.core:SendStatusLost(unitName, "alert_RaidDebuff_Dose")
	end
end

function GridStatusRaidDebuff:EventHandler(ts, event, srcguid, srcname, srcflg, dstguid, dstname, dstflg, ...)
	if 
		(event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_APPLIED_DOSE" or event == "SPELL_AURA_REMOVED_DOSE")
		and bit_band(dstflg, 0x00006868) == 0
	then
		local name, _, type, count = select(2, ...)
		if type ~= "BUFF" then
			self:GainedDebuff(dstname, name, count or 1)
		end
	end
end

function GridStatusRaidDebuff:GainedDebuff(unitName, debuffName, count)
	self:Debug("Gained Debuff",unitName)
	local unitId = RL:GetUnitIDFromName(unitName) or "player"
	local classflg = CFFlgTable[select(2,UnitClass(unitId))] or 0
	local settings = self.db.profile["alert_RaidDebuff_24"]
	local debuff_list = settings.debuff_list[zone]

	if not (settings.enable and debuff_list ~= nil) then
		self.core:SendStatusLost(name, "alert_RaidDebuff_24")
		self.core:SendStatusLost(name, "alert_RaidDebuff_Dose")
		return
	end

	for _, debuff in pairs(debuff_list) do
		if debuff.enable and is_active(debuff.owner) and bit_band(debuff.cf, classflg) > 0 then
			if debuffName == debuff.name then
				if debuff.dose then
					self:StatusSender(unitName, nil, debuff, count,settings.priority, settings.color)
				else
					self:StatusSender(unitName, debuff, nil, nil,settings.priority, settings.color)
				end
				break
			end
		end
	end
end

function GridStatusRaidDebuff:UpdateUnit(unitId)
	self:Debug("Updating Unit",unitName)
	if unitId == nil then return end

	local name = UnitName(unitId)
	local classflg = CFFlgTable[select(2,UnitClass(unitId))] or 0
	local settings = self.db.profile["alert_RaidDebuff_24"]
	local debuff_list = settings.debuff_list[zone]

	if not (settings.enable and debuff_list ~= nil) then
		self.core:SendStatusLost(name, "alert_RaidDebuff_24")
		self.core:SendStatusLost(name, "alert_RaidDebuff_Dose")
		return
	end

	local temp_debuff = nil
	local temp_dose = nil
	local cnt_dose = 0

	for _, debuff in pairs(debuff_list) do
		if debuff.enable and is_active(debuff.owner) and bit_band(debuff.cf, classflg) > 0 then
			if temp_debuff ~= nil and temp_dose ~= nil then break end
			local idx,cnt = Aura:UnitHasDebuff(unitId, debuff.name)
			if idx then
				if debuff.dose then
					temp_dose = debuff
					cnt_dose = cnt
				else
					temp_debuff = debuff
				end
			end
		end
	end

	self:StatusSender(name, temp_debuff, temp_dose, cnt_dose, settings.priority, settings.color)
end

function GridStatusRaidDebuff:GetLinkTooltip(link)
	if link then
		GameTooltip:SetHyperlink(link)
		local t = ""

		for i=1,GameTooltip:NumLines() do
			local tText = getglobal("GameTooltipTextLeft" .. i)
			local cr, cg, cb, ca = tText:GetTextColor()
			cr = string.format("%x",cr * 255)
			cg = string.format("%x",cg * 255)
			cb = string.format("%x",cb * 255)
			ca = string.format("%x",ca * 255)

			if string.len(cr) < 2 then
				cr = "0"..cr
			end

			if string.len(cg) < 2 then
				cg = "0"..cg
			end

			if string.len(cb) < 2 then
				cb = "0"..cb
			end

			if string.len(ca) < 2 then
				ca = "0"..ca
			end
			t = t.."\n|c"..ca..cr..cg..cb..tText:GetText()
		end

		return t
	end
end
